import os, sys, subprocess
import verifier

class Evaluator:
    def __init__(self, algorithm, path):
        self.path = path
        self.algorithm = algorithm
        if os.path.isdir(path):
            self.filenames = [os.path.join(path, fn) for fn in next(os.walk(path))[2]]
        else:
            self.filenames = [path]

    def evaluate_single(self, input_filename):
        output_filename = os.path.join("tmp", os.path.basename(input_filename) + ".tww")
        if os.path.exists(output_filename): 
            os.remove(output_filename)
        print("=============================================================")
        p = subprocess.Popen([
            "/bin/bash",
            "-c",
            f"./build/{self.algorithm} < {input_filename} 1> {output_filename}"
        ])
        p.wait()
        print("=============================================================")
        d,n,max_deg = verifier.check_solution(input_filename, output_filename)
        return d,n,max_deg

    def evaluate(self):
        results = []
        for filename in self.filenames:
            d,n,max_deg = self.evaluate_single(filename)
            results.append((d, n, filename, max_deg))
        print("\n===========================SUMMARY===========================")
        print("algorithm: ", self.algorithm)
        print("testcases: ", self.path)
        print("score, d, n, max_deg, instance")
        avg_score = 0
        for (d,n,filename,max_deg) in results:
            instance = os.path.basename(filename)
            score = 0 if (d == None) else (1 - d / max_deg)
            avg_score += score
            print(f"{score:.3f}", d, n, max_deg, instance)
        avg_score /= len(results)
        print(f"\nmean score: {avg_score}")

if __name__ == '__main__':
    evaluator = Evaluator(sys.argv[1], sys.argv[2])
    evaluator.evaluate()
